from time import sleep
from datetime import datetime
from threading import Thread
from pcaspy import SimpleServer, Driver

from .channels import PV, Image
from .context import Context
from .pvinfoset import PVInfoSet
from .managed import delete_managed


INITIAL_PVDB = { #TODO make wait_time caput-able
    "START_TIME" : {
        "type": "string"
    }
}



class DefaultDriver(Driver): #TODO: caget on START_TIME crashes without this subclass?!
    pass



class MorIOC(Context):

    def __init__(self, prefix, DriverClass=DefaultDriver, wait_time=0.1):
        print("init")
        prefix = prefix.upper()
        prefix = prefix if prefix.endswith(":") else prefix + ":"

        self.prefix = prefix
        self.DriverClass = DriverClass
        self.wait_time = wait_time

        self.server = SimpleServer()
        self.server.createPV(prefix, INITIAL_PVDB)
        self.driver = DriverClass()

        now = datetime.now()
        self.driver.setParam("START_TIME", str(now))
        self.driver.updatePVs()

        self.running = False
        self.thread = Thread(target=self.run, daemon=True)
        self.thread.start()


    def run(self):
        self.running = True
        while self.running:
            self.server.process(self.wait_time)

    def stop(self):
        self.running = False
        wait_time = self.wait_time * 1.1 # wait a bit longer for the last process() call to end
        sleep(wait_time)


    def serve(self, **kwargs):
        pvis = PVInfoSet(kwargs, parse_string=False)
        pvdb = pvis.pvdb()
        self.createPV(pvdb)
        pvvs = pvis.values()
        self.setParams(**pvvs)


    def host(self, **kwargs):
        pvis = PVInfoSet(kwargs, parse_string=True)
        pvdb = pvis.pvdb()
        self.createPV(pvdb)


    def createPV(self, pvdb):
        if not pvdb:
            return
        print("new PVs", pvdb)
        current_pvdb = self.driver.pvDB #TODO crash due to keyerror otherwise
        self.server.createPV(self.prefix, pvdb) #TODO if pv name already contains ":", don't prepend self.prefix?!
        self.driver = self.DriverClass()
        self.driver.pvDB.update(current_pvdb)

    def deletePVs(self, pvdb):
        for name in pvdb:
            self.deletePV(name)

    def deletePV(self, name):
        name = name.upper()
        delete_managed(self.prefix, name)

    def setParams(self, **kwargs):
        for name, value in kwargs.items():
            name = name.upper()
            self.driver.setParam(name, value)
        self.driver.updatePVs()

    def setParam(self, name, value):
        name = name.upper()
        self.driver.setParam(name, value)
        self.driver.updatePVs()

    def get(self, name):
        name = name.upper()
        return self.driver.getParam(name)


    def PV(self, *args, **kwargs):
        return PV(self, *args, **kwargs)

    def Image(self, *args, **kwargs):
        return Image(self, *args, **kwargs)



