from .pvinfo import PVInfo
from .managed import is_managed, get_managed_dtype


class PVInfoSet(set):

    def __init__(self, data, parse_string=True):
        super().__init__(
            PVInfo(*args, parse_string=parse_string) for args in data.items()
        )

    def values(self):
        return make_values(self)

    def pvdb(self):
        return make_pvdb(self)



def make_pvinfos(data, parse_string=True):
    return [PVInfo(*args, parse_string=parse_string) for args in data.items()]


def make_values(pvinfos):
    return {pvi.name: pvi.value for pvi in pvinfos}


def make_pvdb(pvinfos):
    pvdb = {}
    for pvi in pvinfos:
        name = pvi.name

        if is_managed(name):
            dtype = pvi.dtype
            managed_dtype = get_managed_dtype(name)
            if dtype != managed_dtype:
                print(f"dtype for {name} changed:", managed_dtype, "->", dtype) #TODO what should be done if this happens?
#                server.deletePV(name) #TODO ???
            continue # no need to re-create

        print("will create", name)
        pvdb[name] = pvi.to_dict()

    return pvdb



